from django.core.urlresolvers import reverse
from tests.test_admin import APIAuthenticatedTestCase


class APICategoriesEndpoint(APIAuthenticatedTestCase):

    def test_get_view_authorized(self):

        response = self.client.get(reverse("test-view"), **self.header)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["msg"], "Hello World!")

    def test_get_view_unauthorized(self):

        response = self.client.get(reverse("test-view"))

        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.data["detail"], "Authentication credentials were not provided.")



import uuid
from django.db import models


class APIKey(models.Model):

    class Meta:
        verbose_name_plural = "API Keys"
        ordering = ['-created']

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    name = models.CharField(max_length=50, unique=True)
    key = models.CharField(max_length=40, unique=True)

    def __str__(self):
        return self.name



from rest_framework import permissions
from rest_framework_api_key.models import APIKey


class HasAPIAccess(permissions.BasePermission):
    message = 'Invalid or missing API Key.'

    def has_permission(self, request, view):
        api_key = request.META.get('HTTP_API_KEY', '')
        return APIKey.objects.filter(key=api_key).exists()



from django.contrib import admin
from django.contrib import messages
from rest_framework_api_key.models import APIKey
from rest_framework_api_key.helpers import generate_key


class ApiKeyAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'created', 'modified')

    fieldsets = (
        ('Required Information', {'fields': ('name',)}),
        ('Additional Information', {'fields': ('key_message',)}),
    )
    readonly_fields = ('key_message',)

    search_fields = ('id', 'name',)

    def has_delete_permission(self, request, obj=None):
        return False

    def key_message(self, obj):
        if obj.key:
            return "Hidden"
        return "The API Key will be generated once you click save."

    def save_model(self, request, obj, form, change):
        if not obj.key:
            obj.key = generate_key()
            messages.add_message(request, messages.WARNING, ('The API Key for %s is %s. Please note it since you will not be able to see it again.' % (obj.name, obj.key)))
        obj.save()

admin.site.register(APIKey, ApiKeyAdmin)



from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from rest_framework_api_key.models import APIKey
from rest_framework_api_key.helpers import generate_key


class LoggedInAdminTestCase(TestCase):

    USERNAME = "martymcfly"
    EMAIL = "marty@mcfly.com"
    PASSWORD = "password"

    def setUp(self):
        super(LoggedInAdminTestCase, self).setUp()

        self.user = User.objects.create_superuser(self.USERNAME, self.EMAIL, self.PASSWORD)
        self.assertTrue(self.user.is_active)
        self.assertTrue(self.user.is_superuser)
        self.client.login(username=self.USERNAME, password=self.PASSWORD)


class APIAuthenticatedTestCase(TestCase):

    APP_NAME = 'Project Tests'

    def setUp(self):
        self.app_key = APIKey.objects.create(name=self.APP_NAME, key=generate_key())
        self.header = {'HTTP_API_KEY': self.app_key.key}


class AdminTestCase(LoggedInAdminTestCase, APIAuthenticatedTestCase):
    def setUp(self):
        super(AdminTestCase, self).setUp()
        self.add_app_url = reverse('admin:rest_framework_api_key_apikey_add')

    def test_admin_create_app_access(self):
        response = self.client.get(self.add_app_url)
        self.assertEqual(response.status_code, 200)

    def test_admin_create_app(self):
        self.assertEqual(APIKey.objects.all().count(), 1)

        response = self.client.post(self.add_app_url, data={"name": "Hello"}, follow=True)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Please note it since you will not be able to see it again.")
        self.assertContains(response, "was added successfully.")

        self.assertEqual(APIKey.objects.all().count(), 2)

    def test_admin_change_app(self):
        self.change_app_url = reverse('admin:rest_framework_api_key_apikey_change', args=(self.app_key.id,))

        response = self.client.get(self.change_app_url,)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Hidden")



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'django-rest-framework-api-key'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

INSTALLED_APPS = [
    # Django Apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.staticfiles',

    # External Packages
    "rest_framework",
    "rest_framework_api_key",

    # Test apps
    "tests"
]

REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework_api_key.permissions.HasAPIAccess',
    )
}

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = 'tests.urls'

STATIC_URL = '/static/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='APIKey',
            fields=[
                ('id', models.UUIDField(editable=False, serialize=False, default=uuid.uuid4, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(unique=True, max_length=50)),
                ('key', models.CharField(unique=True, max_length=40)),
            ],
            options={
                'ordering': ['-created'],
                'verbose_name_plural': 'API Keys',
            },
        ),
    ]



from django.conf.urls import include, url
from django.contrib import admin
from tests.views import TestView


urlpatterns = [

    url(r'^admin/', include(admin.site.urls)),
    url(r'^test/$', TestView.as_view(), name="test-view"),

]



from setuptools import find_packages, setup

setup(
    name='drfapikey',
    version=__import__('rest_framework_api_key').__version__,
    author="Emmanouil Konstantinidis",
    author_email="manos@iamemmanouil.com",
    packages=find_packages(),
    include_package_data=True,
    url="https://github.com/ekonstantinidis/django-rest-framework-api-key",
    license='BSD',
    description="Authenticate Web APIs made with Django REST Framework",
    long_description=open("README.md").read(),
    install_requires=[],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.7",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
    ],
)



from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView


class TestView(APIView):
    """
    A dummy view used only for testing.
    """

    def get(self, request):
        return Response({"msg": "Hello World!"}, status=status.HTTP_200_OK)



from django.test import TestCase
from rest_framework_api_key.models import APIKey


class ModelTestCase(TestCase):

    def setUp(self):
        super(ModelTestCase, self).setUp()

    def test_admin_create_object(self):
        self.assertEqual(APIKey.objects.all().count(), 0)
        APIKey.objects.create(name="Hello World", key="helloworld")
        self.assertEqual(APIKey.objects.all().count(), 1)
